/*------------------------------------------------------------------------
    File        : WebApp.TP.BL.Trip.cls
    Author(s)   : marc
    Created     : 07-03-2018 13:32:11
    Notes       : Customized BE instance with SPJFP features available.
  ----------------------------------------------------------------------*/
@program FILE(name="TripBE.cls", module="AppServer").
@openapi.openedge.export FILE(type="REST", executionMode="singleton", useReturnValue="false", writeDataSetBeforeImage="false").
@progress.service.resource FILE(name="WebApp.TP.BL.Trip", URI="/TP/BL/TripBE", schemaName="dsTrip", schemaFile="").

USING Progress.Lang.*.
USING Progress.Json.ObjectModel.*.
USING WebApp.System.Session.*.
USING WebApp.System.Util.*.
USING WebApp.TP.Util.*.

BLOCK-LEVEL ON ERROR UNDO, THROW.

CLASS WebApp.TP.BL.TripBE
      INHERITS WebApp.System.BusinessLogic.BusinessEntity:

  {"WebApp/TP/BL/Trip.i"}

  DEFINE DATA-SOURCE srcTrip FOR aa-tp_trip.

  /*-------------------------------------------------------- ----------------------
          Purpose: Prepare the dataset and skipped field lists.
          Notes:
  ------------------------------------------------------------------------------*/
  CONSTRUCTOR PUBLIC TripBE():

    DEFINE VAR hDataSourceArray AS HANDLE    NO-UNDO EXTENT 1.
    DEFINE VAR cSkipListArray   AS CHARACTER NO-UNDO EXTENT 1.
    DEFINE VAR lUseCompanyScope AS LOGICAL   NO-UNDO EXTENT 1.
    DEFINE VAR cUseIndex        AS CHARACTER NO-UNDO EXTENT 1.

    SUPER(DATASET dsTrip:HANDLE).

    /* Data Source for each table in dataset. Should be in table order as defined in DataSet */
    hDataSourceArray[1] = DATA-SOURCE srcTrip:HANDLE.

    /* Skip-list entry for each table in dataset. Should be in temp-table order as defined in DataSet */
    /* Each skip-list entry is a comma-separated list of field names, to be ignored in create statement */
    cSkipListArray[1] = ?.

    /* Then 'companycode' will automaticly addded to the Where Clause by default and save on submit
       Use lUseCompanyScope = NO to ignore this behavour */
    lUseCompanyScope[1] = ?.
    cUseIndex[1] = ?.

    THIS-OBJECT:ProDataSource     = hDataSourceArray.
    THIS-OBJECT:SkipList          = cSkipListArray.
    THIS-OBJECT:spUseCompanyScope = lUseCompanyScope.
    THIS-OBJECT:spUseIndex        = cUseIndex.

    

    THIS-OBJECT:Validator         = NEW WebApp.TP.BL.TripValidator().
    THIS-OBJECT:UIController      = NEW WebApp.TP.BL.TripUIController().

    THIS-OBJECT:FilterTypes = "default,ablFilter,kendoFilter".

  END CONSTRUCTOR.

  /*------------------------------------------------------------------------------
    Purpose:  Define calculated fields
    Notes:    @param phBuffer buffer to current record
  ------------------------------------------------------------------------------*/
  METHOD OVERRIDE PROTECTED VOID DataCalculate( INPUT phBuffer AS HANDLE  ):
    /* Softpak Converted Code Start */

    phBuffer::calc_driver_name = /*------------------------------------------------------------------------------
  Purpose: return drivername from transportorder 
    Notes:  
------------------------------------------------------------------------------*/
METHOD PRIVATE CHARACTER GetDriver
  ( INPUT pcCompanycode AS CHAR, INPUT piOrderSeqnr AS INT ) :

  DEF VAR cReturnValue AS CHAR NO-UNDO.

  DEF BUFFER b-aa-tp_supplier_order  FOR aa-tp_supplier_order.
  DEF BUFFER b-aa-tp_supplier_driver FOR aa-tp_supplier_driver.

  IF piOrderSeqnr <> 0
  THEN DO:

    FOR EACH b-aa-tp_supplier_order WHERE
             b-aa-tp_supplier_order.companycode    = pcCompanycode AND
             b-aa-tp_supplier_order.tp_order_seqnr = piOrderSeqnr NO-LOCK,
       FIRST b-aa-tp_supplier_driver WHERE
             b-aa-tp_supplier_driver.companycode        = b-aa-tp_supplier_order.companycode AND
             b-aa-tp_supplier_driver.supplier_addressnr = b-aa-tp_supplier_order.supplier_addressnr AND
             b-aa-tp_supplier_driver.seqnr              = b-aa-tp_supplier_order.driver_seqnr NO-LOCK:
        
      cReturnValue = b-aa-tp_supplier_driver.name + 
                     (IF b-aa-tp_supplier_driver.number_plate <> ""
                      THEN " / " + number_plate 
                      ELSE "").  

    END.           

  END.

  RETURN cReturnValue .

  END METHOD.

  /*------------------------------------------------------------------------------
     Purpose: Get total count of records, based on a filter string.
     Notes:   @param filter Filter reseved from front-end
              @param numRecs Number of records in this table
              @param exact result count exact or guess (count maximum 1 seconds)
  ------------------------------------------------------------------------------*/
  @openapi.openedge.export(type="REST", useReturnValue="false", writeDataSetBeforeImage="false").
  @progress.service.resourceMapping(type="REST", operation="invoke", URI="/count?filter=~{filter~}", alias="count", mediaType="application/json").
  METHOD PUBLIC VOID CountRecords( INPUT filter AS CHARACTER, OUTPUT numRecs AS INTEGER, OUTPUT exact AS LOGICAL):

    SUPER:Count(INPUT filter, OUTPUT numRecs, OUTPUT exact).

  END METHOD. /* CountRecords */

  /*------------------------------------------------------------------------------
   Purpose: Combine Field sensitivity, values and mandatory fields in a single call from the frontend
   Notes: @param pcInvokeField invloked field
          @param dsTrip prodataset received from front-end
          @param dsFieldState prodataset with mandatory fields & UI Control (Sensitivity & Field Values)
  ------------------------------------------------------------------------------*/
  @openapi.openedge.export(type="REST", useReturnValue="false", writeDataSetBeforeImage="true").
  @progress.service.resourceMapping(type="REST", operation="invoke", URI="/fieldState", alias="fieldState", mediaType="application/json").
  METHOD PUBLIC VOID FieldStateTrip( INPUT  invokeField AS CHARACTER, INPUT newRow AS LOGICAL, INPUT-OUTPUT DATASET dsTrip, OUTPUT DATASET dsFieldState ):


    DEFINE VARIABLE hDataSet AS HANDLE NO-UNDO.
    ASSIGN hDataSet = DATASET dsTrip:HANDLE.

    TEMP-TABLE ttTrip:TRACKING-CHANGES = TRUE.

    SUPER:FieldState(INPUT invokeField,
                     INPUT newRow,
                     INPUT-OUTPUT DATASET-HANDLE hDataSet,
                     OUTPUT DATASET dsFieldState ).

    FINALLY:
      TEMP-TABLE ttTrip:TRACKING-CHANGES = FALSE.
    END FINALLY.

  END METHOD.


  /*------------------------------------------------------------------------------
    Purpose: Get one or more records, based on a filter string
    Notes: @param filter Filter received from font-end.
           @param  dsTrip  proDataset with result of request
  ------------------------------------------------------------------------------*/
  @openapi.openedge.export(type="REST", useReturnValue="false", writeDataSetBeforeImage="true").
  @progress.service.resourceMapping(type="REST", operation="read", URI="?filter=~{filter~}", alias="", mediaType="application/json").
  @openapi.openedge.method.property(name="mappingType", value="SPJFP").
  @openapi.openedge.method.property(name="capabilities",value="kendoFilter,top,skip,id,orderBy").
  METHOD PUBLIC VOID ReadTrip ( INPUT filter AS CHARACTER, OUTPUT DATASET dsTrip ):

    SUPER:ReadData(filter).
    THIS-OBJECT:PostFetchRecord(filter).

  END METHOD. /* Read */

  /*------------------------------------------------------------------------------
    Purpose: Submit one or more records (with create, update, delete action).
    Notes: @param dsTrip proDataset to submit
  ------------------------------------------------------------------------------*/
  @openapi.openedge.export(type="REST", useReturnValue="false", writeDataSetBeforeImage="true").
  @progress.service.resourceMapping(type="REST", operation="submit", URI="/submit", alias="submit", mediaType="application/json").
  METHOD PUBLIC VOID SubmitTrip ( INPUT-OUTPUT DATASET dsTrip ):

    DEFINE VARIABLE hDataSet AS HANDLE NO-UNDO.
    ASSIGN hDataSet = DATASET dsTrip:HANDLE.

    IF NOT THIS-OBJECT:ValidateData   (INPUT DATASET dsTrip BY-REFERENCE) THEN LEAVE.
    IF NOT THIS-OBJECT:PreCommitLogic (INPUT DATASET dsTrip BY-REFERENCE) THEN LEAVE.

    /* USE BY-REFERENCE https://knowledgebase.progress.com/articles/Article/businessentity-not-returning-prods-clientid */
    SUPER:Submit(INPUT-OUTPUT DATASET-HANDLE hDataSet BY-REFERENCE).

    IF NOT THIS-OBJECT:PostCommitLogic(INPUT DATASET dsTrip BY-REFERENCE) THEN LEAVE.

  END METHOD. /* Submit */

  /*------------------------------------------------------------------------------
          Purpose: Provide defaults for a new record.
          Notes: @param dsTrip proDataset with new records to create
  ------------------------------------------------------------------------------*/
  @openapi.openedge.export(type="REST", useReturnValue="false", writeDataSetBeforeImage="false").
  @progress.service.resourceMapping(type="REST", operation="invoke", URI="/new", alias="new", mediaType="application/json").
  METHOD PUBLIC VOID NewRecord ( INPUT-OUTPUT DATASET dsTrip ):

  END METHOD. /* NewRecord */

  /************************************************************************************************/
  /***************************  Business Entity specific methods **********************************/
  /************************************************************************************************/
  /*------------------------------------------------------------------------------
    Purpose: Validate data before create, update, or delete.
    Notes:  @param dsTrip proDataset to validate
    ------------------------------------------------------------------------------*/
  METHOD PROTECTED LOGICAL ValidateData (INPUT DATASET dsTrip):

   DEFINE VARIABLE lErrors    AS LOGICAL NO-UNDO.
   DEFINE VARIABLE oValidator AS WebApp.TP.BL.TripValidator NO-UNDO.

   IF NOT CAN-FIND(ttTrip)  AND NOT CAN-FIND(bttTrip)
   THEN LEAVE.

   oValidator = CAST(THIS-OBJECT:Validator, WebApp.TP.BL.TripValidator).
   oValidator:Validate(INPUT DATASET dsTrip BY-REFERENCE, OUTPUT lErrors).

   RETURN NOT lErrors. /* Succeed */

  END METHOD. /* ValidateData */

  /*------------------------------------------------------------------------------
    Purpose: Any PreCommit logic for create, update, or delete.
    Notes:
    ------------------------------------------------------------------------------*/
  METHOD PROTECTED LOGICAL PreCommitLogic ( INPUT DATASET dsTrip ):

  END METHOD. /* PreCommitLogic*/

  /*------------------------------------------------------------------------------
    Purpose:  Any PostCommit logic for create, update, or delete.
    Notes:
  ------------------------------------------------------------------------------*/
  METHOD PROTECTED LOGICAL PostCommitLogic ( INPUT DATASET dsTrip):

  END METHOD. /* PostCommitLogic */

  /*------------------------------------------------------------------------------
    Purpose: Perform any post-read logic.
    Notes:
  ------------------------------------------------------------------------------*/
  METHOD PROTECTED VOID PostFetchRecord ( INPUT filter AS CHARACTER ):

  END METHOD. /* PostFetchRecord */

  /************************************************************************************************/
  /*************************************  Custom Methods  *****************************************/
  /************************************************************************************************/
  /* Softpak Converted Code Start */
  /* Softpak Converted Code End */


END CLASS.
